/****************************************************
//	AshaTej Software
//
//	the worm segment behavior
*****************************************************/
Segment = function() {
	var self = this;			// shortcut for this
	var overlap = 0.8;			// how much to overlap segments
	var length;					// the length of a segment
	var width;					// the width of a segment
	var frequency = 0;			// frequency of sine wave
	
	this.onAddToScene = function(params) {
		console.log('segment added to scene');
		var options = params || {};
		length = options.length || 100;
		width = options.width || 20;
		frequency = Math.random() * 3;		// seed so segments pulse at variable times
		
		var segmentSprite = new Sprite(
				{
					layer: wade.app.GAME_LAYER,
					autoResize: false,
					image: "./images/segment.jpg",
					alwaysDraw: true,
					pixelShader: wade.app.getSegmentShader(),
					pixelShaderUniforms: {lightDir: "vec3"},
					properties: {lightDir: [0.,0.,0.]},
					size: {x: length,y: width}
				});
		
		self.owner.addSprite(segmentSprite);
	};	// end onAddtoScene
	
	// get the actual position of the front connection spot
	this.getFrontOffsetPosition = function() {
		var o = self.getFrontOffst();
		var p = self.owner.getPosition();
		return {x:p.x+o.x,y:p.y+0,y};
	};	// end getFrontOffsetPosition
	
	// get the actual position of the back connection spot
	this.getBackOffsetPosition = function() {
		var o = self.getBackOffset();
		var p = self.owner.getPosition();
		return {x:p.x+o.x,y:p.y+o.y};
	};	// end getBackOffsetPosition
	
	// get the front connection spot
	// simply the overlap distance times half the length 
	// and correcting the distance based on the rotation
	this.getFrontOffset = function() {
		var p = {x:0,y:0};
		var r = self.owner.getRotation();
		p.x = overlap*length/2*Math.cos(r);
		p.y = overlap*length/2*Math.sin(r);
		return p;
	};	// end getFrontOffset
	
	// get the back connection spot
	// simply the overlap distance times half the length 
	// and correcting the distance based on the rotation
	this.getBackOffset = function() {
		var p = {x:0,y:0};
		var r = self.owner.getRotation();
		p.x = -1*overlap*length/2*Math.cos(r);
		p.y = -1*overlap*length/2*Math.sin(r);
		return p;
	};	// end getBackOffset
	
	// on update pulse the sides of the worm segment
	this.onUpdate = function() {
		self.pulse();
	};	// end onUpdate
	
	// simply vary the width of the segment sinusoidally (is that a word?)
	this.pulse = function() {
		var sp = self.owner.getSprite();
		sp.setSize(length,width+width/2*Math.sin(frequency));
		if (frequency > 2*Math.PI) {frequency = 0; }
		else ( frequency += 0.01)
	};	// end pulse
	
};	// end Segment

// For Chrome Debugging
//@ sourceURL=segment5.js