/****************************************************
//	Ashatej Software
//	July 2016
//
//	Worm
//	Step 5
//
*****************************************************/

//----------------------------------------------------------------
//					Wade global functions
//----------------------------------------------------------------

App = function() {
	this.GAME_LAYER = 6;				// the game layer 
	this.BACKGROUND_LAYER = 7;			//  background look in game.wsc
	
	this.GAME_WIDTH = 920;				// the games screen width
	this.GAME_HEIGHT = 608;				// the games screen height
	
	// game wide variables
	var self = this;					// to shorten wade.app
	
	var worm = [];						// to hold our worm segments
	
	var bgShaderString = '';			// the string to store the background shader
	var segmentShaderString = '';		// the string to store the worm segment shader
	
	//----------------------------------------------------------------
	//					Wade global functions
	//----------------------------------------------------------------
	
	/****************************************************
	// 	Wade global load function
	//	Load scripts, json data files, images and sounds
	*****************************************************/
	this.load = function() {
		console.log('wade global load');
		wade.setLoadingBar(true,{x:0,y:160},'#FFFFFF', '#333366');
		wade.setLoadingImages('./images/ashatej_banner.jpg');
		
		// load script behaviors
		wade.loadScript('./segment5.js')
		
		// game images
		wade.loadImage('./images/bg.jpg');
		wade.loadImage('./images/segment.jpg');
		
		self.loadShaders();
	};	// end load

	/****************************************************
	// 	Wade global init function
	//	set the min and max screen sizes based on constants
	//	set the window mode to full
	// 	remove the loading images
	//	start the  demo
	*****************************************************/
	this.init = function() {
		console.log('wade global init');
		wade.setMinScreenSize(self.GAME_WIDTH,self.GAME_HEIGHT);
		wade.setMaxScreenSize(self.GAME_WIDTH,self.GAME_HEIGHT);
		wade.setWindowMode('full');
		wade.setClickTolerance(15);
		 
		self.removeLoadingImage();
		self.startDemo();
	};	// end init
	
	// remove the ashatej logo and start the game
	this.removeLoadingImage = function() {
		$(".loadingImage_class").remove();
	};	// end removeLoadingImage
	
	// get shader source with jquery ajax
	this.loadShaders = function() {
		console.log('load Shaders');
		// load the background shader
		$.get('./backgroundShader.fs',function(data) {
				bgShaderString = data;
				bgShaderString = bgShaderString.replace(/(\/\*([\s\S]*?)\*\/)|(\/\/(.*)$)/gm, '');
		});
		
		// load the segment shader
		$.get('./segmentShader.fs',function(data) {
				segmentShaderString = data;
				segmentShaderString = segmentShaderString.replace(/(\/\*([\s\S]*?)\*\/)|(\/\/(.*)$)/gm, '');
		});
	};	// end loadShaders
	
	// to access the worm segment shader from the segment behavior
	this.getSegmentShader = function() { return segmentShaderString; };
	
	//----------------------------------------------------------------
	//					Game Management functions
	//----------------------------------------------------------------
	
	this.startDemo = function() {
		console.log('start demo')
		self.createBackground();
		self.createSegment();
	};	// end startDemo
	
	//----------------------------------------------------------------
	//					game objects / world creation functions
	//----------------------------------------------------------------
	
	// create the background
	this.createBackground = function() {
		var backgroundSprite = new Sprite(
				{
					layer: self.BACKGROUND_LAYER,
					autoResize: false,
					image: './images/bg.jpg',
					alwaysDraw: true,
					pixelShader: bgShaderString,
					size: {x: self.GAME_WIDTH,y: self.GAME_HEIGHT}
				});
		var background = new SceneObject(backgroundSprite);
		wade.addSceneObject(background);
	};	// end createBackground
	
	// create a single segment and push it into the worm array
	this.createSegment = function() {
		for (var i=0; i<12; i++) {
			var segment = new SceneObject(0,Segment);
			wade.addSceneObject(segment,true);
			worm.push(segment);
		}
	};	// end createSegment
	
	// move the head - worm[0] to the mouse position
	// move remaining segments to tail of pervious segment
	this.moveWorm = function(wp) {
		var s = worm[0];								// get the head of the worm
		var pos = s.getPosition();						// get the head position
		var dx = wp.x - pos.x;							// calculate delta x
		var dy = wp.y - pos.y;							// calculate delta y
		s.setRotation(Math.atan2(dy,dx));				// rotate based on angle
		var np = s.getBehavior().getFrontOffset();		// get head front node
		s.moveTo(wp.x-np.x,wp.y-np.y);					// move head offset position to mouse position
		
		// now do what you did for the head, except, the new position
		// is the tail position of the prior node in the array
		for (var i=1; i<worm.length; i++) {
			var z = worm[i];							
			var myP = z.getPosition();
			var target = worm[i-1].getBehavior().getBackOffsetPosition();
			var dx = target.x - myP.x;
			var dy = target.y - myP.y;
			z.setRotation(Math.atan2(dy,dx));
			var fo = z.getBehavior().getFrontOffset();
			z.moveTo(target.x-fo.x,target.y-fo.y);
		}
	};	// end moveWorm
	
	
	//----------------------------------------------------------------
	//					Mouse & Input functions
	//----------------------------------------------------------------
	
	// on mouse move, convert to world coordinates and then send those to moveWorm
	this.onMouseMove = function(eventData) {
		var sToW = wade.screenPositionToWorld(self.GAME_LAYER,eventData.screenPosition);
		self.moveWorm(sToW);
	};	// end onMouseMove
	
};	// end App

// For Chrome Debugging
//@ sourceURL=step5.js